#!/usr/bin/python
# -*- coding: utf_8 -*-
"""La idea es mantener un diccionario de secciones, donde cada valor se tiene otro diccionario. Éste segundo diccionario tiene como clave una opción y como valor, el valor asignado a la opción con ="""
import re

__comment = '\s*((\;|\#).*)?$'
__valid_id = '([A-Za-z_]\w*)'
# Si tiene comillas el value es el grupo 3 y las comillas son el 2 y el 4
# sino el value es el grupo 5
__valid_value = '((\"|\')([^\"\']*)(\"|\')|([^\s\#\;]*))'
__valid_section_name = '([\w\.\:\-_]*)'
sectionRE = re.compile('^\[\s*' + __valid_section_name + '[\s]*\]' + __comment)
optionRE = re.compile('^' + __valid_id + '\s*=\s*' + __valid_value + __comment)


import ConfigParser

class IniParser:
	"""Parser de archivos .ini orientado a la manipulación del lts.conf. Por defecto pasa los nombres
de sección a minúscula y los nombres de las opciones a mayúscula, dejando los valores de las opciones
sin modificar. Estos comportamientos se puede modificar con set_sections_transformation, set_options_transformations y optionxform respectivamente.

Atención: las transformaciones de los nombres de sección y opción solo se hacen en
add_section y set (al leer el archivo con read y readfp también se los transforma
ya que se usan los métodos antes nombrados para llenar la estructura de datos).

Eso significa que el algoritmo es case insensitive sólo al cargar y leer el archivo, cuando se consultan
valores con get* o con has_section, es responsabilidad del programador pasar el valor transformado a la
representación interna de la instancia (se puede usar sectionsTransformation() y optionsTransformation() para esto).

Los comentarios son ignorados en las lecturas y descartados en las escrituras."""
	def __init__(self):
		self.__sections = dict()
		self.__default_name = self.sectionxform('default')
		self.__sections[self.__default_name] = dict()
	# Interfaz propia
	def sectionxform(self, section):
		"""Establece que función aplicar a los nombres de sección al leerlos o escribirlos
por defecto los devuelve en minúsculas."""
		return section.lower()

	def valuexform(self, value):
		"""Establece que función aplicar a los valores de opción al leerlos o escribirlos
por defecto no lo modifica."""
		return value

	# Métodos auxiliares
	def __readValidSectionName(self, line):
		"""Intenta parsear una línea en busca de una sección válida, si la encuentra retorna su nombre
sino retorna None (la línea debe comenzar con "[")"""
		global sectionRE
		match = sectionRE.match(line)
		if match != None:
			return match.group(1)
		return None
	def __readValidOptionPair(self, line):
		"""Intenta parsear una línea en busca de una opción válida, si la encuentra retorna una tupla
con de la forma (opción, valor), sino retorna None, si detecta un error de sintáxis en el encomillado
del valor lanza una excepción."""
		global optionRE
		match = optionRE.match(line)
		if match != None:
			groups = match.groups()
			if groups[5] == None:
				# El value tiene comillas
				if groups[2] != groups[4]:
					# Las comillas simples y dobles no coinciden
					raise ConfigParser.ParsingError(str(groups))
				return (groups[0], '"' + groups[3].strip() + '"')
			else:
				# El value no tiene comillas
				return (groups[0], groups[5])
		return None				

	# Interfaz de configparser.ConfigParser, vamos a ver hasta donde la implementamos
	def defaults(self):
		"Return a dictionary containing the instance-wide defaults." # No implementado
		pass
	def sections(self):
		"Return a list of the sections available; DEFAULT is not included in the list."
		return [ key for key in self.__sections if key != self.__default_name]

	def all_sections(self):
		"Return a list of the sections available."
		return [ key for key in self.__sections ]

	def add_section(self, section):
		"""Add a section named section to the instance. If a section by the given name already exists, DuplicateSectionError is raised. If the name DEFAULT (or any of it’s case-insensitive variants) is passed, ValueError is raised."""
		section = self.sectionxform(section)
		if section == self.__default_name:
			return # La sección default se puede agregar N veces
		if self.has_section(section):
			raise ConfigParser.DuplicateSectionError(section)
		self.__sections[section] = dict()

	def has_section(self, section):
		"""Indicates whether the named section is present in the configuration. The DEFAULT section is not acknowledged."""
		return self.__sections.has_key(self.sectionxform(section))

	def options(self, section):
		"Returns a list of options available in the specified section."
		return [ option for option in self.__sections[self.sectionxform(section)]]

	def has_option(self, section, option):
		"If the given section exists, and contains the given option, return True; otherwise return False."
		return self.__sections[self.sectionxform(section)].has_key(self.optionxform(option))

	def read(self, filenames):
		"""Attempt to read and parse a list of filenames, returning a list of filenames which were successfully parsed. If filenames is a string or Unicode string, it is treated as a single filename. If a file named in filenames cannot be opened, that file will be ignored. This is designed so that you can specify a list of potential configuration file locations (for example, the current directory, the user’s home directory, and some system-wide directory), and all existing configuration files in the list will be read. If none of the named files exist, the ConfigParser instance will contain an empty dataset. An application which requires initial values to be loaded from a file should load the required file or files using readfp() before calling read() for any optional files:
    import ConfigParser, os
    
    config = ConfigParser.ConfigParser()
    config.readfp(open('defaults.cfg'))
    config.read(['site.cfg', os.path.expanduser('~/.myapp.cfg')])
"""
		ok = []
		for f in filenames:
			try:
				ini = open(f, 'r')
				self.readfp(ini)
			except:
				continue
			ok.append(f)
		return ok	

	def readfp(self, fp, filename=None):
		"""Read and parse configuration data from the file or file-like object in fp (only the readline() method is used). If filename is omitted and fp has a name attribute, that is used for filename; the default is <???>.""" # ¿¿¿Qué???
		lastSection = self.__default_name # Un truquito, todo lo que esté al principio fuera de una sección
		# está adentro de Default
		for line in fp:
			l = line.lstrip()
			if l == '' or l.startswith('#') or l.startswith(';'):
				# Es un comentario
				continue
			section = self.__readValidSectionName(l)
			if section != None:
				# Es una sección sintácticamente válida
				self.add_section(section)
				lastSection = section
				continue
			option = self.__readValidOptionPair(l)
			if option != None:
				# Es una opción sintácticamente válida
				self.set(lastSection, option[0], option[1])
				continue
			# Si no es comentario, sección u opción es un error de sintáxis
			raise ConfigParser.ParsingError(line)

	def get(self, section, option):
		"Get an option value for the named section."
		section = self.sectionxform(section)
		option = self.optionxform(option)
		if not self.has_section(section):
			raise ConfigParser.NoSectionError(section)
		if not self.has_option(section, option):
			raise ConfigParser.NoOptionError(option, section)
		return self.valuexform(self.__sections[section][option])

	def getint(self, section, option):
		"A convenience method which coerces the option in the specified section to an integer."
		return int(self.get(section, option), 10)

	def getfloat(self, section, option):
		"A convenience method which coerces the option in the specified section to a floating point number."
		return float(self.get(section, option))

	def getboolean(self, section, option):
		"""A convenience method which coerces the option in the specified section to a Boolean value. Note that the accepted values for the option are "1", "yes", "true", and "on", which cause this method to return True, and "0", "no", "false", and "off", which cause it to return False. These string values are checked in a case-insensitive manner. Any other value will cause it to raise ValueError."""
		value = self.get(section, option).lower()
		return value == 'y' or value == 'true' or value == '1' or value == 'on'

	def items(self, section):
		"Return a list of (name, value) pairs for each option in the given section."
		section = self.sectionxform(section)
		return [ (name, self.__sections[section][name]) for name in self.__sections[section] ]
	def set(self, section, option, value):
		"""If the given section exists, set the given option to the specified value; otherwise raise NoSectionError. While it is possible to use RawConfigParser (or ConfigParser with raw parameters set to true) for internal storage of non-string values, full functionality (including interpolation and output to files) can only be achieved using string values."""
		if not self.has_section(section):
			raise ConfigParser.NoSectionError(section)
		self.__sections[self.sectionxform(section)][self.optionxform(option)] = \
			str(self.valuexform(value))

	def __shouldIgnoreSection(self, section):
		"""Return true if the section shouldn't be writen with the write() method, in this class returns true if the section is empty because LTSP5 client scripts can fail if there is any empty section in lts.conf. If you want another behaviour you should override this method."""
		if len(self.__sections[section]) == 0:
			# The section is empty
			return True
		return False

	def write(self, fileobject):
		"""Write a representation of the configuration to the specified file object. This representation can be parsed by a future read() call."""
		for section in self.__sections:
			if self.__shouldIgnoreSection(section):
				continue
			fileobject.write('[' + section + ']\n')
			for option in self.__sections[section]:
				fileobject.write('\t' + option + '=' + self.__sections[section][option] + '\n')

	def remove_option(self, section, option):
		"""Remove the specified option from the specified section. If the section does not exist, raise NoSectionError. If the option existed to be removed, return True; otherwise return False."""
		del self.__sections[self.sectionxform(section)][self.optionxform(option)]

	def remove_section(self, section):
		"""Remove the specified section from the configuration. If the section in fact existed, return True. Otherwise return False."""
		section = self.sectionxform(section)
		if self.__sections.has_key(section):
			self.__sections[section].clear()
			del self.__sections[section]
			return True
		return False
	def optionxform(self, option):
		"""Transforms the option name option as found in an input file or as passed in by client code to the form that should be used in the internal structures. The default implementation returns a upper-case version of option; subclasses may override this or client code can set an attribute of this name on instances to affect this behavior.

    You don’t necessarily need to subclass a IniParser to use this method, you can also re-set it on an instance, to a function that takes a string argument. Setting it to str, for example, would make option names case sensitive:

    cfgparser = ConfigParser()
    ...
    cfgparser.optionxform = str

	Note that when reading configuration files, whitespace around the option names are stripped before optionxform() is called."""
		return option.upper()
#prueba al módulo para ver si funciona

if __name__ == '__main__':
	parser = IniParser()
	parser.readfp(open('/opt/ltsp/i386/etc/lts.conf', 'r'))
	for each in parser.sections():
		print each + " => " + repr(parser.options(each))
	parser.write(open('/tmp/parseado', 'w'))
	print parser.items('default')

class MinIniParser(IniParser, object):
	def __init__(self):
		super(MinIniParser, self).__init__()
		
	def optionxform(self,option):
		"""Transforms the option name option as found in an input file or as passed in by client code to the form that should be used in the internal structures. The default implementation returns a upper-case version of option; subclasses may override this or client code can set an attribute of this name on instances to affect this behavior.

    You don’t necessarily need to subclass a IniParser to use this method, you can also re-set it on an instance, to a function that takes a string argument. Setting it to str, for example, would make option names case sensitive:

    cfgparser = ConfigParser()
    ...
    cfgparser.optionxform = str

	Note that when reading configuration files, whitespace around the option names are stripped before optionxform() is called. 
	Se sobreescribe el m{etodo de IniParser para que no setee en mayusculas las opciones"""
		return option
